/*
 * Created on Jan 15, 2005
 */
package net.kildenpedersen.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

/**
 * A flexible ArrayList. 
 * Allows setting and adding of arbitrary indexed positions. 
 * Will automatically expand list with <code>null</null> if index is less than size.
 * 
 * @author Nils Kilden-Pedersen
 */
public final class ExpandingArrayList extends ArrayList {
	
	/**
	 * @inheritDoc
	 */
	public ExpandingArrayList() {
		super();
	}

	/**
	 * @inheritDoc
	 * @param c
	 */
	public ExpandingArrayList(Collection c) {
		super(c);
	}

	/**
	 * @inheritDoc
	 * @param initialCapacity
	 */
	public ExpandingArrayList(int initialCapacity) {
		super(initialCapacity);
	}
	
	private void adjust(int index) {
		int difference = (index - size()) + 1;

		if (difference < 1) {
			return;
		}

		if (difference == 1) {
			add(null);
		} else {
			addAll(Collections.nCopies(difference, null));
		}
	}
	
	/**
	 * Add element at index, and expand list if necessary.
	 * @param index
	 * @param element
	 * @see java.util.List#add(int, java.lang.Object)
	 */
	public void add(int index, Object element) {
		adjust(index);
		super.add(index, element);
	}

	/**
	 * @inheritDoc
	 * @see java.util.ArrayList#get(int)
	 */
	public Object get(int index) {
		if (index >= size()) {
			return null;
		}
		return super.get(index);
	}

	/**
	 * Set element at index, and expand list if necessary.
	 * @param index
	 * @param element
	 * @return replaced element
	 * @see java.util.List#set(int, java.lang.Object)
	 */
	public Object set(int index, Object element) {
		adjust(index);
		return super.set(index, element);
	}
	
	/**
	 * @inheritDoc
	 * @see java.util.ArrayList#addAll(int, java.util.Collection)
	 */
	public boolean addAll(int index, Collection c) {
		adjust(index);
		return super.addAll(index, c);
	}
}
