package net.sourceforge.orbroker;

import net.kildenpedersen.reflect.Reflector;

/*
 * Created on Mar 19, 2004
 */

/**
 * ArgumentList contains a method or constructors arguments.
 * @author Nils Kilden-Pedersen
 */
final class ArgumentList extends TypedList {
	
	static final ArgumentList NO_ARG;
	static {
		NO_ARG = new ArgumentList(0);
		NO_ARG.close();
	}
	
	private Class[] parmTypes;

	/**
	 * Constructor.
	 * @param size Initial size
	 */
	ArgumentList(int size) {
		super(size);
	}
	
	private ValueType getValueType(int idx) {
		return (ValueType) super.get(idx);
	}
	
	/**
	 * Add a {@link ValueType} to the list.
	 * @param type The ValueType.
	 */
	void add(ValueType type) {
		super.add(type);
	}
	
	/**
	 * Returns the {@link java.lang.Class} array.
	 * No synchronization is used, as it is assumed that
	 * concurrent threads will produce the same result.
	 * @return argument types
	 */
	Class[] getArgumentTypes() {
		assert isClosed() :
			Reflector.getClassName(getClass()) + 
				" must be closed when using this operation.";

		if (this.parmTypes == null) {
			Class[] parmClasses = new Class[size()];
			for (int i = 0; i < parmClasses.length; i++) {
				parmClasses[i] = getValueType(i).getType();
			}
			this.parmTypes = parmClasses;
		}
		
		return this.parmTypes;
	}

	/**
	 * Get the argument values as array.
	 * @param row
	 * @return array of argument values
	 */
	Object[] getArgumentValues(ResultRow row) {
		assert isClosed() :
			Reflector.getClassName(getClass()) + 
				" must be closed when using this operation.";
		
		Object[] parmValues = new Object[size()];
		for (int i = 0; i < parmValues.length; i++) {
			parmValues[i] = getValueType(i)
				.getValue(row);
		}
		return parmValues;
	}
}
