/*
 * Created on Feb 24, 2004
 */
package net.sourceforge.orbroker;

/**
 * A column definition.
 * @author Nils Kilden-Pedersen
 */
final class Column extends ValueType {
	
	private static boolean isPositiveInteger(String column) {
		for (int i = 0; i < column.length(); i++) {
			if (column.charAt(i) < '0' || column.charAt(i) > '9') {
				return false;
			}
		}
		return true;
	}

	private final String columnName;
	private final int index;
	
	/**
	 * Constructor.
	 * @param columnName
	 * @param requiredClass
	 */
	Column(String columnName, Class requiredClass) {
		super(requiredClass);
		String column;
		int idx;
		if (isPositiveInteger(columnName)) {
			idx = Integer.parseInt(columnName);
			column = null;
		} else {
			column = columnName;
			idx = 0;
		}
		this.columnName = column;
		this.index = idx;
	}
	
	String getName() {
		return this.columnName != null ? this.columnName : String.valueOf(this.index);
	}

	/**
	 * @throws ConfigurationException
	 * @see net.sourceforge.orbroker.ValueType#getValue(ResultRow)
	 */
	Object getValue(ResultRow row) throws ConfigurationException {
		
		int columnIndex;
		if (this.columnName == null) {
			columnIndex = this.index;
		} else {
			columnIndex = row.findColumn(this.columnName);
		}
		return row.getColumnValue(columnIndex, this.getType());
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
	    return "Column " + this.columnName;
	}
}
