/*
 * Created on Oct 20, 2004
 */
package net.sourceforge.orbroker;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * The ConnectionContext class is a container for both text replacements and SQL parameters.
 * @author Nils Kilden-Pedersen
 */
final class ConnectionContext {
    private Parameters parameters = new Parameters();

    private final TextReplacements textReplacements = new TextReplacements();
    
    void addContext(ConnectionContext context) {
        this.textReplacements.setAll(context.textReplacements);
        this.parameters.addParameters(context.parameters);
    }

	Map copyParameters() {
        return new HashMap(this.parameters.toMap());
    }



	Map getParametersAsMap() {
        return this.parameters.toMap();
    }
    
    Object getParameterValue(String path) {
        return this.parameters.getLeafValue(path);
    }
    
    TextReplacements getTextReplacements() {
        return this.textReplacements;
    }
    
    /**
     * Remove parameter.
     * @param name parameter to remove
     */
    void removeParameter(String name) {
        this.parameters.removeParameters(name);
    }
    
    /**
     * @param name
     * @param value
     */
    void setParameter(String name, Object value) {
        this.parameters.addParameter(name, value);
    }
    
    void setParameters(Parameters newParms) {
        this.parameters = newParms;
    }
    
    /**
     * @param name
     * @param text
     */
    void setTextReplacement(String name, String text) {
        this.textReplacements.setProperty(name, text);
    }
    
    void setTextReplacements(Properties textReplacements) {
        this.textReplacements.setAll(textReplacements);
    }
    
    /**
     * @param textReplacements
     */
    void setTextReplacements(TextReplacements textReplacements) {
        this.textReplacements.setAll(textReplacements);
    }
    
    /**
     * @inheritDoc
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return 
        	"Replacements: " + this.textReplacements.toString() + 
        	"\nParameters: " + this.parameters.toString();
    }
}
