/*
 * Created on Aug 13, 2004
 */
package net.sourceforge.orbroker;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Map;

import freemarker.template.Template;

/**
 * @author Nils Kilden-Pedersen
 */
final class FreeMarkerStatement extends DynamicStatement {

    private final Template template;
	
	/**
	 * Constructor.
	 * @param id
	 * @param statement
	 * @param resultObjectDef
	 */
	FreeMarkerStatement(String id, String statement, ResultObjectDefinition resultObjectDef) {
		super(id, resultObjectDef);
		try {
            this.template = new Template(id, new StringReader(statement));
        } catch (IOException e) {
            throw new ConfigurationException(e);
        }
	}

    /**
     * @inheritDoc
     * @throws BrokerException
     * @see net.sourceforge.orbroker.Statement#getRuntimeStatement(net.sourceforge.orbroker.Parameters, java.util.Map)
     */
    ImmutableSQL getRunnableSQL(ConnectionContext context) throws BrokerException {
		Map rootMap = context.copyParameters();
		StringWriter writer = new StringWriter();
		try {
            this.template.process(rootMap, writer);
        } catch (Exception e) {
            throw buildFailedParsingException(e);
        }
        logBrokerStatement(writer.getBuffer(), Statement.Type.FREEMARKER);
        return new SQLParser(writer.getBuffer()).getRunnableStatement(context.getTextReplacements());
    }
}
