package net.sourceforge.orbroker;

import java.util.HashMap;
import java.util.Map;
import net.kildenpedersen.reflect.Reflector;


/*
 * Created on Apr 22, 2004
 *
 */

/**
 * Holding class for parameter values. Based on a Map.
 * @author Nils Kilden-Pedersen
*/
final class Parameters {
    
	private final Map map;
	
	Parameters() {
	    this.map = new HashMap();
	}
	
	Parameters(Parameters source) {
	    this.map = new HashMap(source.map);
	}
	
	void addParameter(String name, Object value) {
		if (name == null) {
			String msg = 
					"No parameter name given";
			throw new NullPointerException(msg);
		}
	    this.map.put(name, value);
	}
	
	void addParameters(Parameters newParameters) {
	    if (!newParameters.map.isEmpty()) {
	        this.map.putAll(newParameters.map);
	    }
	}

    /**
     * Return the leaf value, i.e. the last value in a property tree. Ex.:
     * <code>root.propertyBranch.propertyLeaf</code> will return the value of
     * propertyLeaf after traversing the tree.
     * 
     * @param parameterTree
     * @return leaf value
     */
	Object getLeafValue(String parameterTree) {
		Object value = this.map.get(parameterTree);
		if (value != null) {
		    return value;
		}
		return Reflector.getLeafValue(this.map, parameterTree);
	}
	
	void removeParameters(String path) {
		this.map.remove(path);
	}
	
	/**
	 * Return values as {@link Map}.
	 * @return parameter values in map.
	 */
	Map toMap() {
	    return this.map;
	}
	
	/**
	 * @inheritDoc
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
	    return this.map.toString();
	}

}
