/*
 * Created on Feb 22, 2004
 */
package net.sourceforge.orbroker;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.logging.Level;

/**
 * The query class. Read-only queries can use a Query object. 
 * It wraps a {@link java.sql.Connection Connection}
 * object, and {@link #close()} should be called after use, to avoid
 * open connections, or connection pool leaks.
 * @author Nils Kilden-Pedersen
 * @see net.sourceforge.orbroker.Broker#startQuery()
 * @see net.sourceforge.orbroker.Broker#startQuery(int)
 */
public final class Query extends QueryableConnection {

	Query(Broker owner, Integer isolationLevel) throws BrokerException {
		super(owner, isolationLevel);
	}

	/**
	 * @see net.sourceforge.orbroker.QueryableConnection#getClosedConnectionMessage()
	 */
	protected String getClosedConnectionMessage() {
		return "Query has been closed";
	}

	/**
	 * @see net.sourceforge.orbroker.QueryableConnection#getFinalizeWarning()
	 */
	protected String getFinalizeWarning() {
		return 
		"A Query was not closed. Temporary connection pool leak now closed.";
	}
	
	protected void setConnectionProperties() throws SQLException {
		Connection con = getActiveConnection();
		if (!con.isReadOnly()) {
			con.setReadOnly(true);
		}
	}

    /**
     * @see net.sourceforge.orbroker.BrokerConnection#shouldConnectionClose()
     */
    protected boolean shouldConnectionClose() {
        return true;
    }
	
	/**
	 * Close query. Subsequent closes will be ignored.
	 */
	public void close() {
		if (!isAutoCommit()) {
			try {
				// Call commit to release any resource locks
				getActiveConnection().commit();
			} catch (SQLException e) {
				Broker.log(Level.WARNING, "Commit on Query caused " + e.toString());
			}
		}
		dropConnection();
	}

}