package net.sourceforge.orbroker;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;


/*
 * Created on May 18, 2004
 *
 */

/**
 * A container for Statement and ResultSet.
 * 
 * @author Nils Kilden-Pedersen
 */
final class QueryResult {
	private final Connection connection;
    private final ConnectionContext context;
    private final ResultSet outputParameters;
    private final ResultSet resultSet;
    private final java.sql.Statement sqlStatement;
	
	QueryResult(
			java.sql.Statement stm, 
			ResultSet resultSet, 
			ConnectionContext context) throws SQLException {
        this(stm, resultSet, context, null);
	}
    
    QueryResult(java.sql.Statement stm, ResultSet resultSet, ConnectionContext context, ResultSet outputParameters) throws SQLException {
        this.sqlStatement = stm;
        this.resultSet = resultSet;
        this.context = context;
        this.outputParameters = outputParameters;
		this.connection = stm.getConnection();
    }

    /**
     * @param context
     * @param cs
     */
    private void setOutputParameters() {
        if (this.outputParameters != null) {
	    	ResultRow row = new ResultRow(this.outputParameters, this.context, this.connection);
	        Statement.setValuesOnMap(this.context.getParametersAsMap(), row);
        }
    }

    void close() {
		try {
			this.resultSet.close();
		} catch (SQLException e) {
		    Broker.log(Level.WARNING, e.toString());
		}
		
        setOutputParameters();
        
        try {
			this.sqlStatement.close();
		} catch (SQLException e) {
		    Broker.log(Level.WARNING, e.toString());
		}
	}

	ResultRow getResultRow() {
		return new ResultRow(this.resultSet, this.context, this.connection);
	}

	/**
	 * Get result row positioned at the given start position.
	 * @param startRow
	 *                   row to position to.
	 * @param scrollable
	 *                   true, if result set is scrollable
	 * @return Positioned result row
	 * @throws QueryException
	 */
	ResultRow getResultRow(int startRow, boolean scrollable) throws QueryException {
		if (startRow > 1) {
			try {
				if (scrollable) {
					this.resultSet.absolute(startRow - 1);
				} else {
                    setFetchSize(startRow);
					int currentPosition = 1;
					while (currentPosition < startRow) {
						if (!this.resultSet.next()) {
							break;
						}
						currentPosition++;
					}
				}
			} catch (SQLException e) {
				throw new QueryException(e);
			}
		}

		return getResultRow();
	}

	boolean nextRow() throws QueryException {
		try {
			return this.resultSet.next();
		} catch (SQLException e) {
			throw new QueryException(e);
		}
	}

	void setFetchSize(int rows) {
	    try {
            this.resultSet.setFetchSize(rows);
        } catch (SQLException e) {
            Broker.log(Level.WARNING, e.toString());
        }
	}
}