/*
 * Created on Oct 14, 2004
 */
package net.sourceforge.orbroker;

import java.util.Arrays;

/**
 * @author Nils Kilden-Pedersen
 */
final class ResultKey {
	
	private static final Object[] EMPTY_KEY = new Object[0];
	
    private static int buildHashCode(String name, Object[] values) {
        int hash = 0;
        for (int i = 0; i < values.length; i++) {
            hash = 37 * hash + values[i].hashCode();
        }
        return hash + name.hashCode();
    }
    
	private static String buildToString(String name, Object[] values) {
        StringBuffer buffer = new StringBuffer();
        buffer.append(name).append("(");
        for (int i = 0; i < values.length; i++) {
            if (i > 0) {
                buffer.append(", ");
            }
            buffer.append(values[i]);
        }
        buffer.append(")");
        return buffer.toString();
	}
    private final int hash;
    private final String resultObjectName;
    private final String toString;
    private final Object[] values;
    
    ResultKey(final String resultObjectName) {
    	this.resultObjectName = resultObjectName;
        this.values = EMPTY_KEY;
        this.hash = 0;
        this.toString = buildToString(resultObjectName, this.values);
    }
    
    ResultKey(final String resultObjectName, final Object[] values) {
    	assert (values != null && values.length > 0) :
    		"Invalid value array passed. Is either null or of 0 length.";
    	this.resultObjectName = resultObjectName;
        this.values = values;
        this.hash = buildHashCode(resultObjectName, values);
        this.toString = buildToString(resultObjectName, values);
    }
    
    /**
     * A valid key is one that is not defined as "*".
     * @return true if no key exists
     */
    boolean isValidKey() {
    	return this.values.length > 0;
    }
    
    /**
     * @inheritDoc
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
    	return equals((ResultKey) obj);
    }
    
    /**
     * Compare two keys
     * @param comp
     * @return true, if equal
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(ResultKey comp) {
        return 
			this.resultObjectName.equals(comp.resultObjectName) &&
			Arrays.equals(this.values, comp.values);
    }

    /**
     * @inheritDoc
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.hash;
    }
    
    /**
     * @inheritDoc
     * @see java.lang.Object#toString()
     */
    public String toString() {
    	return this.toString;
    }
}