/*
 * Created on Oct 14, 2004
 */
package net.sourceforge.orbroker;

import java.util.StringTokenizer;


/**
 * @author Nils Kilden-Pedersen
 */
final class ResultKeyDef {
	
    private static Column[] buildKeyColumns(final String delimitedColumns) {
        StringTokenizer tokens = new StringTokenizer(delimitedColumns, " ,;:+\r\n");
        Column[] columns = new Column[tokens.countTokens()];
        for (int i = 0; i < columns.length; i++) {
            String columnName = tokens.nextToken();
            columns[i] = new Column(columnName, Object.class);
        }
        return columns;
    }

    private final Column[] columns;

	private final String resultObjectName;

    ResultKeyDef(final String delimitedColumns, final String resultObjectName) {
        this.resultObjectName = resultObjectName;
        if (delimitedColumns.trim().equals("*")) {
            this.columns = null;
            return;
        }
        this.columns = buildKeyColumns(delimitedColumns);
        if (this.columns.length == 0) {
            throw new ConfigurationException(
                    "No columns given for key.");
        }
    }

    /**
     * Get column values for key columns.
     * If any column is null, return a null array.
     * @param row
     * @return column values
     */
    private Object[] getDefinedColumns(ResultRow row) {
        Object[] values = new Object[this.columns.length];
        for (int i = 0; i < values.length; i++) {
            values[i] = this.columns[i].getValue(row);
            if (values[i] == null) {
                return null;
            }
        }
        return values;
    }

    /**
     * Get key values
     * 
     * @param row
     * @return key or null, if any column was null.
     */
    ResultKey getResultKey(ResultRow row) {
        if (this.columns == null) {
        	return new ResultKey(this.resultObjectName);
        }
        Object[] values = getDefinedColumns(row);
        if (values == null) {
            return null;
        }
        return new ResultKey(this.resultObjectName, values);
    }
    /**
	 * @inheritDoc
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("Key columns:");
		for (int i = 0; i < this.columns.length; i++) {
			buffer.append(" ").append(this.columns[i].getName());
		}
		return buffer.toString();
	}
}