/*
 * Created on Feb 23, 2004
 */
package net.sourceforge.orbroker;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import net.kildenpedersen.reflect.Reflector;


/**
 * @author Nils Kilden-Pedersen
 */
class ResultObjectFactoryMethod extends ResultObjectInstantiator {
	
	private static Method buildStaticMethod(Class type, String name, ArgumentList parameters) {
		parameters.close();
		Class[] parmClasses = parameters.getArgumentTypes();
		Method method = Reflector.getAccessibleMethod(type, name, parmClasses); 
		if (method == null) {
			String msg = "Unknown factory method: " + 
			Reflector.getClassName(type) + ". " + name + parameters.toString();
			throw new ConfigurationException(msg);
		}
		if (!Modifier.isStatic(method.getModifiers())) {
		    String msg = "Factory method not static: " + 
		    Reflector.getClassName(type) + "." + name + parameters.toString();
			throw new ConfigurationException(msg);
		}
		return method;
	}
	
	ResultObjectFactoryMethod(Class type, String methodName, ArgumentList parameters) {
		super(buildStaticMethod(type, methodName, parameters), parameters);
	}

	private Method getMethod() {
		return (Method) getMember();
	}

    /**
     * @inheritDoc
     * @see net.sourceforge.orbroker.ResultObjectInstantiator#copy(Class)
     */
    ResultObjectInstantiator copy(Class forType) {
        return new ResultObjectFactoryMethod(
                forType, getMethod().getName(), getArgumentList());
    }

	/**
	 * Get new instance.
	 * @param row
	 * @return result object
	 */
	Object newInstance(ResultRow row) {
		Object[] parmObjects = this.getArgumentList().getArgumentValues(row);
		try {
		    return getMethod().invoke(null, parmObjects);
		} catch (Exception e) {
			String msg = "Cannot construct " + Reflector.getClassName(getType()) + 
				" using factory method: " + getMethod().toString();
			throw new ReflectionException(msg, e);
		}
	}
}
