/*
 * Created on Jun 16, 2004
 */
package net.sourceforge.orbroker;

import java.sql.ParameterMetaData;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

/**
 * @author Nils Kilden-Pedersen
 */
final class ResultParameterMetaData implements ResultSetMetaData {

	private final ParameterMetaData metaData;
	private final String[] parameterNames;
	
	/**
	 * Constructor.
	 * @param metaData
	 * @param parmNames
	 */
	public ResultParameterMetaData(ParameterMetaData metaData, String[] parmNames) {
		this.metaData = metaData;
		this.parameterNames = parmNames;
	}
	/**
	 * @see java.sql.ResultSetMetaData#getColumnCount()
	 */
	public int getColumnCount() throws SQLException {
		return this.metaData.getParameterCount();
	}

	/**
	 * @see java.sql.ResultSetMetaData#getColumnDisplaySize(int)
	 */
	public int getColumnDisplaySize(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.sql.ResultSetMetaData#getColumnType(int)
	 */
	public int getColumnType(int columnIndex) throws SQLException {
		return this.metaData.getParameterType(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#getPrecision(int)
	 */
	public int getPrecision(int columnIndex) throws SQLException {
		return this.metaData.getPrecision(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#getScale(int)
	 */
	public int getScale(int columnIndex) throws SQLException {
		return this.metaData.getScale(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#isNullable(int)
	 */
	public int isNullable(int columnIndex) throws SQLException {
		return this.metaData.isNullable(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#isAutoIncrement(int)
	 */
	public boolean isAutoIncrement(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.sql.ResultSetMetaData#isCaseSensitive(int)
	 */
	public boolean isCaseSensitive(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.sql.ResultSetMetaData#isCurrency(int)
	 */
	public boolean isCurrency(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.sql.ResultSetMetaData#isDefinitelyWritable(int)
	 */
	public boolean isDefinitelyWritable(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.sql.ResultSetMetaData#isReadOnly(int)
	 */
	public boolean isReadOnly(int columnIndex) throws SQLException {
		return true;
	}

	/**
	 * @see java.sql.ResultSetMetaData#isSearchable(int)
	 */
	public boolean isSearchable(int columnIndex) throws SQLException {
		return false;
	}

	/**
	 * @see java.sql.ResultSetMetaData#isSigned(int)
	 */
	public boolean isSigned(int columnIndex) throws SQLException {
		return this.metaData.isSigned(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#isWritable(int)
	 */
	public boolean isWritable(int columnIndex) throws SQLException {
		return false;
	}

	/**
	 * @see java.sql.ResultSetMetaData#getCatalogName(int)
	 */
	public String getCatalogName(int columnIndex) throws SQLException {
		return "[unknown]";
	}

	/**
	 * @see java.sql.ResultSetMetaData#getColumnClassName(int)
	 */
	public String getColumnClassName(int columnIndex) throws SQLException {
		return this.metaData.getParameterClassName(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#getColumnLabel(int)
	 */
	public String getColumnLabel(int columnIndex) throws SQLException {
		return getColumnName(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#getColumnName(int)
	 */
	public String getColumnName(int columnIndex) throws SQLException {
	    return this.parameterNames[columnIndex-1];
	}

	/**
	 * @see java.sql.ResultSetMetaData#getColumnTypeName(int)
	 */
	public String getColumnTypeName(int columnIndex) throws SQLException {
		return this.metaData.getParameterTypeName(columnIndex);
	}

	/**
	 * @see java.sql.ResultSetMetaData#getSchemaName(int)
	 */
	public String getSchemaName(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see java.sql.ResultSetMetaData#getTableName(int)
	 */
	public String getTableName(int arg0) throws SQLException {
		throw new UnsupportedOperationException();
	}

}
