/*
 * Created on Oct 27, 2004
 */
package net.sourceforge.orbroker;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator returned from 
 * {@link net.sourceforge.orbroker.QueryableConnection#selectIterator(String, int) 
 * selectIterator(String, int)}.
 * @author Nils Kilden-Pedersen
 */
final class ResultSetIterator implements Iterator {

    private Boolean hasNext = null;
    private QueryResult queryResult;
    private final ResultRow row;
    private final Statement statement;
    
    /**
     * Constructor.
     * @param queryResult
     * @param statement
     */
    ResultSetIterator(QueryResult queryResult, Statement statement) {
        this.queryResult = queryResult;
        this.statement = statement;
        this.row = this.queryResult.getResultRow();
    }

    synchronized void closeIterator() {
        if (this.queryResult != null) {
            this.queryResult.close();
            this.queryResult = null;
        }
    }

    /**
     * Make sure query result is closed.
     * @inheritDoc
     * @see java.lang.Object#finalize()
     */
    protected void finalize() throws Throwable {
        closeIterator();
        super.finalize();
    }
    
    /**
     * @inheritDoc
     * @see java.util.Iterator#hasNext()
     */
    public boolean hasNext() {
    	if (this.queryResult == null) {
    		return false;
    	}
    	if (this.hasNext == null) {
    		this.hasNext = Boolean.valueOf(this.queryResult.nextRow());
    	}
        if (!this.hasNext.booleanValue()) {
            closeIterator();
        }
        return this.hasNext.booleanValue();
    }
    /**
     * @inheritDoc
     * @see java.util.Iterator#next()
     */
    public Object next() {
        if (this.hasNext == null || !this.hasNext.booleanValue()) {
            throw new NoSuchElementException("Always check hasNext() before calling next().");
        }
        Object result = this.statement.buildResultObject(this.row);
        this.hasNext = null;
        return result;
    }
    /**
     * @inheritDoc
     * @see java.util.Iterator#remove()
     */
    public void remove() {
        throw new UnsupportedOperationException("remove()");
    }

}
