/*
 * Created on Jun 17, 2004
 */
package net.sourceforge.orbroker;

import java.lang.reflect.Field;
import net.kildenpedersen.reflect.Reflector;


/**
 * @author Nils Kilden-Pedersen
 */
final class SetField extends ResultObjectMember implements Setter {
	
	private static Field findField(Class type, String fieldName) {
		Field field = Reflector.getAccessibleField(type, fieldName);
		if (field != null) {
			return field;
		}
		throw new ConfigurationException(
		        "Unknown field '" + fieldName + "' for " + type.toString());
	}
	
	private ValueType valueType;

	/**
	 * Constructor.
	 * @param resultObjectType
	 * @param fieldName
	 * @param valueType
	 */
	SetField(Class resultObjectType, String fieldName) {
		super(findField(resultObjectType, fieldName));
	}

	private Field getField() {
		return (Field) getMember();
	}
	
	/**
	 * Returns the field type.
	 * @return field type
	*/
	Class getFieldType() {
		return getField().getType();
	}
	/**
	 * @param valueType Set valueType.
	 */
	void setValueType(ValueType valueType) {
		this.valueType = valueType;
	}

	/**
	 * Set valueType
	 * @param resultObject
	 * @param row
	 */
	public final void setValue(Object resultObject, ResultRow row) {
		Field setField = getField();
		Object parmValue = this.valueType.getValue(row);
		try {
			setField.set(resultObject, parmValue);
		} catch (IllegalAccessException e) {
			String msg = 
				"Cannot set field '" + setField.getName() + 
				"' on " + resultObject.getClass(); 
			throw new ReflectionException(msg, e);
		} catch (RuntimeException e) {
			if (setField.getType().isPrimitive() && parmValue == null) {
				String msg = 
					"Cannot assign 'null' to  primitive field '" + setField.getName() + 
					"' in class '" + Reflector.getClassName(setField.getDeclaringClass()) + "'";
				throw new ReflectionException(msg, e);
			}
			throw e;
		}
	}
}
