/*
 * Created on Mar 19, 2004
  */
package net.sourceforge.orbroker;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;



/**
 * @author Nils Kilden-Pedersen
 */
abstract class SetterMethod extends ArgumentsMember implements Setter {
	protected SetterMethod(Method setMethod) {
		super(setMethod);
	}
	
	private Method getMethod() {
		return (Method) getMember();
	}

	final Class[] getParameterTypes() {
		return getMethod().getParameterTypes();
	}
	/**
	 * Set value
	 * @param resultObject
	 * @param row
	 */
	public final void setValue(Object resultObject, ResultRow row) {
		Method setMethod = getMethod();
		ArgumentList parmList = getArgumentList();
		Object[] parmObjects =parmList.getArgumentValues(row);
		try {
			setMethod.invoke(resultObject, parmObjects);
		} catch (InvocationTargetException e) {
			throw new ReflectionException(e.getTargetException());
		} catch (NullPointerException e) {
			Class[] parmTypes = setMethod.getParameterTypes();
			for (int i = 0; i < parmTypes.length; i++) {
				if (parmTypes[i].isPrimitive() && parmObjects[i] == null) {
					String msg = "Cannot assign 'null' using method '" + 
					setMethod.toString() +
					"' on argument " + (i+1) + " of type '" + parmTypes[i] + "'";
					throw new ReflectionException(msg, e);
				}
			}
			throw e;
		} catch (Exception e) {
			String msg = 
				"Cannot invoke method: " + setMethod; 
			throw new ReflectionException(msg, e);
		}
	}
}
