/*
 * Created on Aug 13, 2004
 */
package net.sourceforge.orbroker;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.Properties;
import java.util.logging.Level;




import net.sourceforge.orbroker.velocity.LogWrapper;
import net.sourceforge.orbroker.velocity.StringResourceLoader;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.context.Context;

/**
 * @author Nils Kilden-Pedersen
 */
final class VelocityStatement extends DynamicStatement {
    
	private static final VelocityEngine ENGINE;
	static {
		ENGINE = new VelocityEngine();
		Properties props = new Properties();
		try {
	        InputStream velocityConfig = ENGINE.getClass().getResourceAsStream("/velocity.properties");
		    if (velocityConfig != null) {
		        props.load(velocityConfig);
		    } else {
		        Broker.log(Level.CONFIG, "Unable to find \"velocity.properties\" file");
		    }
        } catch (IOException e) {
            Broker.log(Level.WARNING, e.toString());
        }
        props.setProperty("runtime.log.logsystem.class", LogWrapper.class.getName());
		props.setProperty("resource.loader", "string");
		props.setProperty("string.resource.loader.class", StringResourceLoader.class.getName());
        try {
			ENGINE.init(props);
		} catch (Exception e) {
			throw new ConfigurationException(e);
		}
	}
    private final Template template;
	
	/**
	 * Constructor.
	 * @param id
	 * @param statement
	 * @param resultObjectDef
	 */
	VelocityStatement(String id, String statement, ResultObjectDefinition resultObjectDef) {
		super(id, resultObjectDef);
        if (statement.indexOf("##") > -1) {
            throw new ConfigurationException("Velocity statements cannot contain '##' for comments with O/R Broker. Use '#* comment *#' instead.");
        }
        try {
            this.template = ENGINE.getTemplate(statement);
        } catch (Exception e) {
            throw new ConfigurationException(e);
        }

	}

	    /**
	     * @inheritDoc
	     * @param parameter
	     * @param replacements
	     * @return runtime statement
	     * @throws BrokerException
	     */
	    ImmutableSQL getRunnableSQL(ConnectionContext context)
	            throws BrokerException {
			Context ctx = new VelocityContext(context.copyParameters());
			StringWriter writer = new StringWriter();
			try {
	            this.template.merge(ctx, writer);
	        } catch (Exception e) {
	            throw buildFailedParsingException(e);
	        }
	        logBrokerStatement(writer.getBuffer(), Statement.Type.VELOCITY);
	        return new SQLParser(writer.getBuffer()).getRunnableStatement(context.getTextReplacements());
	    }
}
