/*
 * Created on Oct 21, 2004
 */

package net.sourceforge.orbroker.exception;
import java.sql.SQLException;

/**
 * The default implementation that uses standard ISO values for SQLState for evaluation.
 * Can be extended to override methods for more database specific evaluation.
 * @author Nils Kilden-Pedersen
 */
public class DefaultExceptionEvaluator implements ExceptionEvaluator {
    
    /**
     * Return SQLState.
     * @param e
     * @return a SQLState code of length 5 or <code>null</code> if no 5 char state was found.
     */
    protected String getState(SQLException e) {
        String state = e.getSQLState();
        if (state != null) {
            state = state.trim().toUpperCase();
            if (state.length() != 5) {
                return null;
            }
        }
        return state;
    }
    
    /**
     * @param source
     * @return 2 char SQLState class or <code>null</code> if such does not exist.
     */
    protected String getStateClass(SQLException source) {
        String state = getState(source);
        if (state != null) {
            return state.substring(0, 2); 
        }
        return null;
    }

    /**
     * @see net.sourceforge.orbroker.exception.ExceptionEvaluator#isConstraint(java.sql.SQLException)
     */
    public boolean isConstraint(SQLException source) {
        String stateClass = getStateClass(source);
        return "23".equals(stateClass);
    }
    
    /**
     * @see net.sourceforge.orbroker.exception.ExceptionEvaluator#isDeadlock(java.sql.SQLException)
     */
    public boolean isDeadlock(SQLException source) {
        String state = getState(source);
        return ("40001".equals(state) || "57033".equals(state));
    }

    /**
     * @see net.sourceforge.orbroker.exception.ExceptionEvaluator#isStaleConnection(java.sql.SQLException)
     */
    public boolean isStaleConnection(SQLException source) {
        String state = getState(source);
        return ("08001".equals(state) || "08003".equals(state));
    }
}
